import React from 'react';
import { WithTranslation } from 'next-i18next';
import { withTranslation } from '../../i18n';

import { WatermarkType, SelectOptionType } from '../../constants/type';
import Button from '../Button';
import Box from '../Box';
import Icon from '../Icon';
import Typography from '../Typography';
import Tabs from '../Tabs';
import Divider from '../Divider';
import ColorSelect from '../ColorSelector';
import SliderWithTitle from '../SliderWithTitle';
import TextBox from '../WatermarkTextBox';
import ImageSelector from '../WatermarkImageSelector';
import PageSelector from '../WatermarkPageSelector';

import {
  Container, Head, Body, Footer, IconWrapper,
} from '../../global/sidebarStyled';

type i18nProps = {
  t: (key: string) => string;
};

type OwnerProps = WatermarkType & {
  onClick: () => void;
  isActive: boolean;
  onSave: () => void;
  onDelete: () => void;
  setDataState: (arg: Record<string, any>) => void;
};

type Props = i18nProps & OwnerProps;

const WatermarkOption: React.FC<Props> = ({
  t,
  onClick,
  type,
  opacity,
  scale = 1,
  rotation,
  textcolor,
  text,
  onSave,
  onDelete,
  setDataState = (): void => {
    // do nothing
  },
}: Props) => (
  <Container>
    <Head>
      <IconWrapper>
        <Icon glyph="left-back" onClick={onClick} />
      </IconWrapper>
      <Typography light>
        {t('watermark')}
      </Typography>
    </Head>
    <Body>
      <Typography variant="subtitle" align="left">
        {t('type')}
      </Typography>
      <Tabs
        options={[
          {
            key: 'text',
            content: t('text'),
            child: (
              <TextBox
                t={t}
                defaultValue={text}
                onChange={(value: string): void => { setDataState({ text: value }); }}
              />
            ),
          },
          {
            key: 'image',
            content: t('image'),
            child: (
              <ImageSelector
                t={t}
                onChange={(data: string): void => { setDataState({ imagepath: data }); }}
              />
            ),
          },
        ]}
        onChange={(option: SelectOptionType): void => { setDataState({ type: option.key }); }}
      />
      {
        type === 'text' ? (
          <>
            <Divider orientation="horizontal" style={{ margin: '20px 0' }} />
            <ColorSelect
              title={t('color')}
              mode="watermark"
              selectedColor={textcolor}
              onClick={(val: string): void => { setDataState({ textcolor: val }); }}
            />
          </>
        ) : null
      }
      <Box mt="20">
        <SliderWithTitle
          title={t('opacity')}
          value={opacity}
          tips={`${opacity}%`}
          onSlide={(val: number): void => { setDataState({ opacity: val }); }}
        />
      </Box>
      <Box mt="20">
        <SliderWithTitle
          title={t('rotate')}
          value={rotation}
          maximum={360}
          tips={`${rotation}°`}
          onSlide={(val: number): void => { setDataState({ rotation: val }); }}
        />
      </Box>
      <Box mt="20">
        <SliderWithTitle
          title={t('scale')}
          value={scale * 100}
          tips={`${Math.round(scale * 100)}%`}
          minimum={50}
          maximum={250}
          onSlide={(val: number): void => { setDataState({ scale: val / 100 }); }}
        />
      </Box>
      <Divider orientation="horizontal" style={{ margin: '20px 0' }} />
      <PageSelector t={t} />
    </Body>
    <Footer>
      <Button appearance="primary" onClick={onSave}>
        {t('save')}
      </Button>
      <Button appearance="danger-link" onClick={onDelete}>
        {t('removeWatermark')}
      </Button>
    </Footer>
  </Container>
);

const translator = withTranslation('sidebar');

type TransProps = WithTranslation & OwnerProps;

export default translator<TransProps>(WatermarkOption);
