import {
  useState, useRef, useCallback, useEffect,
} from 'react';
import { fromEvent } from 'rxjs';
import {
  throttleTime,
} from 'rxjs/operators';

import { getAbsoluteCoordinate } from '../helpers/position';

const defaultTime = 35;

type CursorPosition = {
  x: number | null;
  y: number | null;
  clientX: number | null;
  clientY: number | null;
  screenX: number | null;
  screenY: number | null;
};

type UseCursorPositionType = (
  (throttleTime?: number) => [
    CursorPosition, (element: HTMLElement| SVGSVGElement | SVGCircleElement | null) => void
  ]
);

const initialState = {
  x: null,
  y: null,
  clientX: null,
  clientY: null,
  screenX: null,
  screenY: null,
};

const useCursorPosition: UseCursorPositionType = (time = defaultTime) => {
  const [state, setState] = useState<CursorPosition>(initialState);
  const [
    element, setElement,
  ] = useState<HTMLElement | SVGSVGElement | SVGCircleElement | null>(null);
  const entered = useRef(false);
  const isDown = useRef(false);

  const onMoveEvent = useCallback((e: MouseEvent | Event): void => {
    if (element) {
      const {
        clientX,
        clientY,
        screenX,
        screenY,
      } = e as MouseEvent;
      const coordinate = getAbsoluteCoordinate(element, e as MouseEvent);

      setState(() => ({
        x: coordinate.x,
        y: coordinate.y,
        clientX,
        clientY,
        screenX,
        screenY,
      }));
    }
  }, [element]);

  useEffect(() => {
    if (element) {
      let subscription: any = null;
      const onEnter = (): void => {
        entered.current = true;
      };

      const onLeave = (): void => {
        entered.current = false;
        setState(initialState);
      };

      const onDown = (e: any): void => {
        entered.current = true;
        isDown.current = true;
        onMoveEvent(e);
      };

      subscription = fromEvent(element, 'mousemove').pipe(throttleTime(time)).subscribe(onMoveEvent);

      const onUp = (): void => {
        entered.current = false;
        isDown.current = false;
        setState(initialState);
        if (subscription) subscription.unsubscribe();
      };

      const addEvent = element.addEventListener.bind(element);
      addEvent('mouseenter', onEnter);
      addEvent('mouseleave', onLeave);
      addEvent('mousedown', onDown);
      addEvent('mouseup', onUp);
    }
  }, [element]);

  return [state, setElement];
};

export default useCursorPosition;
